Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 166664)
+++ channels/chan_sip.c	(working copy)
@@ -2478,6 +2478,7 @@
 static int copy_via_headers(struct sip_pvt *p, struct sip_request *req, const struct sip_request *orig, const char *field);
 static void set_destination(struct sip_pvt *p, char *uri);
 static void append_date(struct sip_request *req);
+static void append_notify_custom_headers(struct sip_request *req, struct ast_variable *vars);
 static void build_contact(struct sip_pvt *p);
 static void build_rpid(struct sip_pvt *p);
 
@@ -9729,7 +9730,6 @@
 static int transmit_invite(struct sip_pvt *p, int sipmethod, int sdp, int init)
 {
 	struct sip_request req;
-	struct ast_variable *var;
 	
 	req.method = sipmethod;
 	if (init) {/* Bump branch even on initial requests */
@@ -9790,11 +9790,7 @@
 	add_header(&req, "Supported", SUPPORTED_EXTENSIONS);
 
 	if(p->notify_headers) {
-		char buf[512];
-		for (var = p->notify_headers; var; var = var->next) {
-			ast_copy_string(buf, var->value, sizeof(buf));
-			add_header(&req, var->name, ast_unescape_semicolon(buf));
-		}
+		append_notify_custom_headers(&req, p->notify_headers);
 	}
 	if (p->options && p->options->addsipheaders && p->owner) {
 		struct ast_channel *chan = p->owner; /* The owner channel */
@@ -10237,17 +10233,14 @@
 
 	initreqprep(&req, p, SIP_NOTIFY);
 
-	/* Copy notify vars and add headers */
+	/* Copy notify vars for case if auth request */
 	p->notify_headers = newvar = ast_variable_new("Subscription-State", "terminated", "");
-	add_header(&req, newvar->name, newvar->value);
 	for (var = vars; var; var = var->next) {
-		char buf[512];
 		ast_debug(2, "  Adding pair %s=%s\n", var->name, var->value);
-		ast_copy_string(buf, var->value, sizeof(buf));
-		add_header(&req, var->name, ast_unescape_semicolon(buf));
 		newvar->next = ast_variable_new(var->name, var->value, "");
 		newvar = newvar->next;
 	}
+	append_notify_custom_headers(&req, p->notify_headers);
 
 	if (!p->initreq.headers) { /* Initialize first request before sending */
 		initialize_initreq(p, &req);
@@ -10256,6 +10249,32 @@
 	return send_request(p, &req, XMIT_UNRELIABLE, p->ocseq);
 }
 
+/*! /brief Add custom headers and body to NOTIFY request */
+static void append_notify_custom_headers(struct sip_request *req, struct ast_variable *vars) {
+	struct ast_variable *var;
+	struct ast_str *message_body = ast_str_alloca(500);
+
+	if (!vars) {
+		return;
+	}
+	for (var = vars; var; var = var->next) {
+		char buf[512];
+		ast_copy_string(buf, var->value, sizeof(buf));
+		if (!strcasecmp(var->name, "Message-Body")) {
+			ast_str_append(&message_body, 0, "%s\r\n", buf);
+		} else {
+			add_header(req, var->name, ast_unescape_semicolon(buf));
+		}
+	}
+
+	if (message_body->used != 0) {
+		add_header_contentLength(req, message_body->used);
+		add_line(req, message_body->str);
+	} else {
+		add_header_contentLength(req, 0);
+	}
+}
+
 static int manager_sipnotify(struct mansession *s, const struct message *m)
 {
 	const char *channame = astman_get_header(m, "Channel");
Index: configs/sip_notify.conf.sample
===================================================================
--- configs/sip_notify.conf.sample	(revision 166664)
+++ configs/sip_notify.conf.sample	(working copy)
@@ -2,47 +2,38 @@
 
 [aastra-check-cfg]
 Event=>check-sync
-Content-Length=>0
 
 [aastra-xml]
 Event=>aastra-xml
-Content-Length=>0
 
 ; Linksys
 
 [linksys-cold-restart]
 Event=>reboot_now
-Content-Length=>0
 
 [linksys-warm-restart]
 Event=>restart_now
-Content-Length=>0
 
 ; Polycom
 
 [polycom-check-cfg]
 Event=>check-sync
-Content-Length=>0
 
 ; Sipura
 
 [sipura-check-cfg]
 Event=>resync
-Content-Length=>0
 
 [sipura-get-report]
 Event=>report
-Content-Length=>0
 
 ; snom
 
 [snom-check-cfg]
 Event=>check-sync\;reboot=false
-Content-Length=>0
 
 [snom-reboot]
 Event=>reboot
-Content-Length=>0
 
 ;;
 ;; The following NOTIFY messages are not confirmed to work.
@@ -50,4 +41,3 @@
 
 [cisco-check-cfg]
 Event=>check-sync
-Content-Length=>0
